@@ -32,41 +32,13 @@ class dim20:
     ledger_left = -4
     ledger_right = 14
     accidental_widths = { -2: 12, -1: 7, 0: 5, 1: 8, 2: 8}
-    clef_yoffset = {'violin': linespacing-38,
-                    'treble': linespacing-38,#treble == violin
-                    'subbass': -8-2*linespacing,
-                    'bass': -8-linespacing,
-                    'baritone': -8-3*linespacing,
-                    'varbaritone': -8,
-                    'tenor': -8-2*linespacing,
-                    'alto': -8-linespacing,
-                    'mezzosoprano': -15 + linespacing,
-                    'soprano': -15 + 2 * linespacing,
-                    'french': -38 + 2 * linespacing}
-class dim20tight:
-    linespacing = 8
-    stemlen = linespacing * 3.5
-    xshift = 10
-    col_width = 20
-    first_staff_ypos = 70
-    staff_spacing = linespacing * 6
-    ledger_left = -4
-    ledger_right = 14
-    accidental_widths = { -2: 12, -1: 7, 0: 5, 1: 8, 2: 8}
-    clef_yoffset = {'violin': linespacing-38,
-                    'treble': linespacing-38,#treble == violin
-                    'subbass': -8-2*linespacing,
-                    'bass': -8-linespacing,
-                    'baritone': -8-3*linespacing,
-                    'varbaritone': -8,
-                    'tenor': -8-2*linespacing,
-                    'alto': -8-linespacing,
-                    'mezzosoprano': -15 + linespacing,
-                    'soprano': -15 + 2 * linespacing,
-                    'french': -38 + 2 * linespacing}
+    clef_yoffset = {
+        'G': -38 + 3 * linespacing,
+        'F':  -8 + 3 * linespacing,
+        'C': -15 + 3 * linespacing,
+    }
 
-
-dimentions = {20: dim20, "20-tight": dim20tight}
+dimentions = {20: dim20}
 
 accidental_y_offset = {const.ACCIDENTAL__2: -7,
                        const.ACCIDENTAL__1: -7,
@@ -91,28 +63,17 @@ class Engraver:
 class ClefEngraver(Engraver):
     def __init__(self, timepos, fontsize, clef):
         Engraver.__init__(self, timepos, fontsize)
-        self.m_cleftype = clef
+        self.m_clef = clef
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_clef
     def get_width(self):
         return 25
     def engrave(self, widget, gc, staff_yoffset):
-        fn = {'violin': 'G',
-              'treble': 'G',
-              'subbass': 'F',
-              'bass': 'F',
-              'baritone': 'C',
-              'varbaritone': 'F',
-              'tenor': 'C',
-              'alto': 'C',
-              'mezzosoprano': 'C',
-              'soprano': 'C',
-              'french': 'G'}[self.m_cleftype]
         widget.window.draw_pixbuf(gc,
             gtk.gdk.pixbuf_new_from_file(fetadir+'/feta%i-clefs-%s.xpm' \
-                % (self.m_fontsize, fn)),
+                % (self.m_fontsize, self.m_clef.get_symbol())),
             0, 0, self.m_xpos,
-            dimentions[self.m_fontsize].clef_yoffset[self.m_cleftype] \
+            dimentions[self.m_fontsize].clef_yoffset[self.m_clef.get_symbol()] - self.m_clef.get_stafflinepos() * dimentions[self.m_fontsize].linespacing\
                 + staff_yoffset)
 
 class TimeSignatureEngraver(Engraver):
@@ -219,7 +180,7 @@ class KeySignatureEngraver(Engraver):
             if acc in self.m_accidentals:
                 continue
             type = 0
-            ypos = mpdutils.an_to_ylinepos(acc, self.m_clef)
+            ypos = self.m_clef.an_to_ylinepos(acc)
             dolist.append((0, x, ypos))
             #FIXME see FIXME msg in .get_width
             x += 6
@@ -233,7 +194,7 @@ class KeySignatureEngraver(Engraver):
                 type = 2
             else:
                 type = 1
-            ypos = mpdutils.an_to_ylinepos(acc, self.m_clef)
+            ypos = self.m_clef.an_to_ylinepos(acc)
             dolist.append((type, x, ypos))
             #FIXME see FIXME msg in .get_width
             x += 10
@@ -566,9 +527,7 @@ class RestEngraver(Engraver):
         Engraver.__init__(self, timepos, fontsize)
         self.m_ypos = ypos
         self.m_dots = dur.m_dots
-        self.m_type = {Rat(1, 1): 0, Rat(1, 2): 1, Rat(1,4): 2,
-                        Rat(1, 8): 3, Rat(1, 16): 4, Rat(1, 32): 5,
-                        Rat(1, 64): 6}[dur.get_rat_value()]
+        self.m_type = {1: 0, 2: 1, 4: 2, 8: 3, 16: 4, 32: 5, 64: 6}[dur.m_nh]
     def set_xpos(self, dict):
         self.m_xpos = dict[self.m_timepos].m_music
     def get_width(self):
@@ -586,6 +545,12 @@ class RestEngraver(Engraver):
             0, 0,
             self.m_xpos,
             int(staff_yoffset - my + dim.linespacing*self.m_ypos/2 - 4))
+        for n in range(self.m_dots):
+            widget.window.draw_pixbuf(gc,
+                gtk.gdk.pixbuf_new_from_file(fetadir+'/feta20-dots-dot.xpm'),
+                0, 0,
+                int(self.m_xpos+dim.xshift*(1.5+n/2.0)),
+                -3 + staff_yoffset + dim.linespacing*self.m_ypos/2)
     def get_pixmap_id(self):
         return {1: const.REST_1,
                 2: const.REST_2,
