@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
-"""
+r"""
 REMEMBER: down is positive, up is negative.
 
 All voices begin at the beginning of the staff. It is
@@ -24,8 +24,6 @@ not possible to split a voice in two like in Lilypond.
 The parser will not handle fis and f in the same octave on one stem.
 
 Rules:
- * The clef has to be set in the beginning of the staff, and it cannot
-   be changed.
  * It can be different timesignatures in different staffs.
  * \key has to come before \time
 
@@ -80,6 +78,79 @@ def musicalpitch_relative(first, second):
         n1.m_octave_i += second.m_octave_i
         return n1
 
+class UnknownClefException(_exceptions.MpdException):
+    def __init__(self, clef):
+        _exceptions.MpdException.__init__(self)
+        self.m_clef = clef
+    def __str__(self):
+        return "'%s' is not a valid clef. Maybe a bug in your lessonfile?" % self.m_clef
+
+class Clef(object):
+    # Use these constants to access the data in clefdata.
+    SYM = 0
+    # Which staff line should the clef be on lines 1 to 5. 1 is the lowest line
+    LINE = 1
+    # On which position in the staff is the middle C. 0 is the middle line
+    # in the staff. Positive values are up, negative are down.
+    POS = 2
+    clefdata = {
+            'treble': ('G', 2, -6),
+            'violin': ('G', 2, -6),
+                 'G': ('G', 2, -6),
+                'G2': ('G', 2, -6),
+            'french': ('G', 1, -8),
+            #
+            'subbass': ('F', 5, 8),
+               'bass': ('F', 4, 6),
+                  'F': ('F', 4, 6),
+        'varbaritone': ('F', 3, 4),
+        #
+           'baritone': ('C', 5, 4),
+              'tenor': ('C', 4, 2),
+               'alto': ('C', 3, 0),
+                  'C': ('C', 3, 0),
+       'mezzosoprano': ('C', 2, -2),
+            'soprano': ('C', 1, -4),
+    }
+    octaviation_re = re.compile("(?P<name>[A-Za-z1-9]+)(?P<oct>([_^])(8|15))?$")
+    def __init__(self, clefname):
+        m = self.octaviation_re.match(clefname)
+        if not m:
+            raise UnknownClefException(clefname)
+        if m.group('name') not in self.clefdata:
+            raise UnknownClefException(clefname)
+        try:
+            self.m_octaviation = {'_8': -7, '_15': -14, '^8': 7, '^15': 14,
+                             None: 0}[m.group('oct')]
+        except KeyError:
+            raise UnknownClefException(clefname)
+        self.m_name = m.group('name')
+    def get_symbol(self):
+        return self.clefdata[self.m_name][self.SYM]
+    def get_stafflinepos(self):
+        return self.clefdata[self.m_name][self.LINE]
+    def steps_to_ylinepos(self, steps):
+        return 7-self.clefdata[self.m_name][self.POS] - steps + self.m_octaviation
+    def an_to_ylinepos(self, an):
+        def notename_to_ylinepos(n, clef):
+            n = MusicalPitch.new_from_notename(n)
+            i = n.steps()
+            return self.steps_to_ylinepos(i)
+        if an[-2:] == 'es':
+            l = 3
+            h = -3
+        else:
+            l = 1
+            h = -5
+        i = notename_to_ylinepos(an, self)
+        while i > l:
+            an = an + "'"
+            i =  notename_to_ylinepos(an, self)
+        while i < h:
+            an = an + ","
+            i = notename_to_ylinepos(an, self)
+        return i
+
 
 class TimeSignature:
     def __init__(self, num, den):
@@ -165,7 +236,7 @@ class Score:
                     # stems
                     v = []
                     for n in voice.m_coldict[timepos].m_music.values():
-                        v.append(mpdutils.steps_to_ylinepos(n.m_pitch.steps(), clef))
+                        v.append(clef.steps_to_ylinepos(n.m_pitch.steps()))
                     if v and (voice.m_coldict[timepos].m_music.values()[0].m_duration.m_nh > 1):
                         v.sort()
                         if voice.m_coldict[timepos].m_beaminfo == 'start':
@@ -238,7 +309,7 @@ class Score:
                     for music in voice.m_coldict[timepos].m_music.itervalues():
                         e = staff.needed_accidental(music.m_pitch)
                         if e is not None:
-                            v[mpdutils.steps_to_ylinepos(music.m_pitch.steps(), clef)] = e
+                            v[clef.steps_to_ylinepos(music.m_pitch.steps())] = e
                 if v:
                     se.append(AccidentalsEngraver(timepos, fontsize, v))
                     self.m_timeposdict[timepos].m_accidentals \
@@ -276,7 +347,7 @@ class Score:
                     #   the notehead.
                     nd = {}
                     for mm in voice.m_coldict[timepos].m_music.itervalues():
-                        nd[mpdutils.steps_to_ylinepos(mm.m_pitch.steps(), clef)] = mm
+                        nd[clef.steps_to_ylinepos(mm.m_pitch.steps())] = mm
                     if nd == {}:
                         # nd == {} when there are not noteheads, for example
                         # when there is a rest here.
@@ -319,7 +390,7 @@ class Score:
                             head = const.NOTEHEAD_2
                         else:
                             head = const.NOTEHEAD_1
-                        ylinepos = mpdutils.steps_to_ylinepos(music.m_pitch.steps(), clef)
+                        ylinepos = clef.steps_to_ylinepos(music.m_pitch.steps())
                         se.append(NoteheadEngraver(timepos, fontsize, music.m_shift,
                              ylinepos, head, music.m_duration.m_dots,
                              music.m_pitch.semitone_pitch(),
@@ -345,7 +416,7 @@ class Score:
                         continue
                     for music in voice.m_coldict[timepos].m_music.itervalues():
                         ypos = music.m_pitch.steps()
-                        ypos = mpdutils.steps_to_ylinepos(ypos, clef)
+                        ypos = clef.steps_to_ylinepos(ypos)
                         if up > ypos < -5:
                             up = ypos
                         if down < ypos > 5:
@@ -369,7 +440,7 @@ class Score:
                     sh2 = tie[4].m_shift
                     se.append(TieEngraver(fontsize, tie[0], tie[1],
                                           sh1, sh2,
-                                          mpdutils.steps_to_ylinepos(tie[2][0], clef)))
+                                          clef.steps_to_ylinepos(tie[2][0])))
         #########################################
         xv = {}
         p = 0
@@ -505,7 +576,7 @@ class Staff:
     def add_keysignature(self, timepos, key):
         self.m_coldict[timepos].m_keysignature = key
     def add_clef(self, timepos, clef):
-        self.m_coldict[timepos].m_clef = clef
+        self.m_coldict[timepos].m_clef = Clef(clef)
     def announce_timepos(self, timepos):
         if timepos not in self.m_coldict:
             self.m_coldict[timepos] = StaffColObj()
@@ -815,6 +886,7 @@ class Lexer:
     re_staff = re.compile(r"\s*\\staff", re.UNICODE)
     re_voice = re.compile(r"\s*\\addvoice", re.UNICODE)
     re_clef = re.compile(r"\s*\\clef\s+(\w*)", re.UNICODE)
+    re_clef_quoted = re.compile(r"\s*\\clef\s+\"([A-Za-z1-9]+[_^1-9]*)\"", re.UNICODE)
     re_stem_updown = re.compile(r"\s*(\\stem)(Up|Down|Both)\s+", re.UNICODE)
     re_tuplet_updown = re.compile(r"\s*(\\tuplet)(Up|Down|Both)\s+", re.UNICODE)
     re_relative = re.compile(r"\s*\\relative\s+(([a-zA-Z]+)([',]*))", re.UNICODE)
@@ -908,6 +980,10 @@ class Lexer:
         if m:
             self.m_idx += len(m.group())
             return self.RELATIVE, MusicalPitch.new_from_notename(m.group(1))
+        m = self.re_clef_quoted.match(self.m_string, self.m_idx)
+        if m:
+            self.m_idx += len(m.group())
+            return self.CLEF, m.group(1)
         m = self.re_clef.match(self.m_string, self.m_idx)
         if m:
             self.m_idx += len(m.group())
