@@ -16,18 +16,6 @@
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
 """
-When running on a non win32 platform, the soundcard.play_wav_file
-functions has to be set after initialisation:
-
-import soundcard
-
-def play_wav(filename):
-    os.system("aplay %s" % filename)
-
-soundcard.initialise_external_midiplayer(....)
-soundard.play_wav_file = play_wav
-----
-
 Api used to export exercises to midi file
 =========================================
 soundcard.start_export(export_to_filename)
@@ -50,20 +38,17 @@ midiexporter = None
 # we are exporting using midiexporter.
 _saved_synth = None
 
-def win32_play_wav_file(filename):
-    winsound.PlaySound(filename, winsound.SND_FILENAME | winsound.SND_ASYNC)
-
-def linux_play_wav_file(filename):
-    print "play_wav_file not set. Not playing:", filename
-
-def linux_play_midi_file(fileame):
-    print "play_midi_file not set. Not playing:", filename
-
 if sys.platform == 'win32':
     import winsound
-    play_wav_file = win32_play_wav_file
-else:
-    play_wav_file = linux_play_wav_file
+
+def play_mediafile(typeid, filename):
+    import src.cfg, src.osutils
+    if sys.platform == 'win32' and typeid == 'wav':
+        winsound.PlaySound(filename, winsound.SND_FILENAME | winsound.SND_ASYNC)
+    else:
+        src.osutils.run_external_program(
+            src.cfg.get_string("sound/%s_player" % typeid), "", filename)
+
 
 def initialise_winsynth(synthnum, verbose_init=0):
     import winsynth
