@@ -728,12 +728,12 @@ class LessonbasedGui(Gui):
         Show the answer in the g_music_displayer if we have one, if not
         use a new window.
         """
-        if not self.m_t.m_P.get_question()['music'].is_displayable():
-            return
         if 'vmusic' in self.m_t.m_P.get_question():
             varname = 'vmusic'
         else:
             varname = 'music'
+        if not self.m_t.m_P.get_question()[varname].is_displayable():
+            return
         self.display_music(varname)
     def display_music(self, varname):
         """
