@@ -29,17 +29,6 @@ from osutils import *
 import reportlib
 import const
 
-def linux_play_wav_file(filename):
-    if not cfg.get_string('sound/wav_player'):
-        raise ExecutableDoesNotExist('')
-    run_external_program(cfg.get_string('sound/wav_player'), "", filename)
-
-def linux_play_midi_file(filename):
-    run_external_program(cfg.get_string('sound/midi_player'), "", filename)
-
-soundcard.play_wav_file = linux_play_wav_file
-soundcard.play_midi_file = linux_play_midi_file
-
 def check_rcfile():
     """See default.config for rcfileversion values, meanings and
     a description of how to add config variables.
