@@ -48,14 +48,12 @@ class Teacher(abstract.Teacher):
                                       const.QSTATUS_GIVE_UP)):
             return self.ERR_PICKY
         self.m_P.select_random_question()
-        self.m_correct = [self.m_P.get_cname() == -1,
-                          self.m_P.get_inversion() == -1,
-                          self.m_P.get_toptone() == -1]
+        self.m_solved = {}.fromkeys(self.m_P.m_props.keys(), False)
         self.q_status = const.QSTATUS_NEW
         return self.OK
     def give_up(self):
         self.q_status = const.QSTATUS_GIVE_UP
-    def guess_type(self, t):
+    def guess_property(self, property_name, value):
         """
         GUI guarantees that this method will not be called after it has
         been guessed correct once.
@@ -65,54 +63,15 @@ class Teacher(abstract.Teacher):
         return ALL_CORRECT if all parts of the question is correct.
         """
         assert self.q_status == const.QSTATUS_NEW
-        if t == self.m_P.get_cname():
-            self.m_correct[0] = 1
-            if self.m_correct == [1, 1, 1]:
+        if value == self.m_P.get_question()[property_name].cval:
+            self.m_solved[property_name] = True
+            if False not in self.m_solved.values():
                 self.q_status = const.QSTATUS_SOLVED
                 return self.ALL_CORRECT
             return self.CORRECT
         else:
             return 0
-    def guess_inversion(self, t):
-        """
-        GUI guarantees that this method will not be called after it has
-        been guessed correct once.
-        
-        return 0 if this was wrong guess.
-        return CORRECT if this question is correct.
-        return ALL_CORRECT if all parts of the question is correct.
-        """
-        assert self.q_status == const.QSTATUS_NEW
-        if t == self.m_P.get_inversion():
-            self.m_correct[1] = 1
-            if self.m_correct == [1, 1, 1]:
-                self.q_status = const.QSTATUS_SOLVED
-                return self.ALL_CORRECT
-            return self.CORRECT
-        else:
-            return 0
-        self.m_correct[1] = 1
-        return t == self.m_P.get_inversion()
-    def guess_toptone(self, t):
-        """
-        GUI guarantees that this method will not be called after it has
-        been guessed correct once.
 
-        return 0 if this was wrong guess.
-        return CORRECT if this question is correct.
-        return ALL_CORRECT if all parts of the question is correct.
-        """
-        assert self.q_status == const.QSTATUS_NEW
-        if t == self.m_P.get_toptone():
-            self.m_correct[2] = 1
-            if self.m_correct == [1, 1, 1]:
-                self.q_status = const.QSTATUS_SOLVED
-                return self.ALL_CORRECT
-            return self.CORRECT
-        else:
-            return 0
-        self.m_correct[2] = 1
-        return t == self.m_P.get_toptone()
 
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
@@ -120,40 +79,11 @@ class Gui(abstract.LessonbasedGui):
         ################
         # practise_box #
         ################
-        hbox = gu.bHBox(self.practise_box)
+        self.g_hbox = hbox = gu.bHBox(self.practise_box)
         hbox.set_spacing(gu.PAD)
         spacebox = gtk.HBox()
         hbox.pack_start(spacebox)
 
-        table = gtk.Table(4, 5, 0)
-        hbox.pack_start(table, False)
-        hsep = gtk.HSeparator()
-        table.attach(hsep, 0, 5, 1, 2, xoptions=gtk.FILL,
-                     yoptions=gtk.FILL, xpadding=0, ypadding=gu.PAD_SMALL)
-        self.g_vsep1 = gtk.VSeparator()
-        table.attach(self.g_vsep1, 1, 2, 0, 4,
-                     xoptions=0, xpadding=gu.PAD)
-        self.g_vsep2 = gtk.VSeparator()
-        table.attach(self.g_vsep2, 3, 4, 0, 4,
-                     xoptions=0, xpadding=gu.PAD)
-
-        table.attach(gtk.Label(_("Chord type")), 0, 1, 0, 1,
-                     xoptions=gtk.FILL, yoptions=0)
-        self.g_chordtype_box = gtk.VBox()
-        table.attach(self.g_chordtype_box, 0, 1, 2, 3, xoptions=gtk.FILL)
-
-        self.g_inversion_label = gtk.Label(_("Inversion"))
-        table.attach(self.g_inversion_label, 2, 3, 0, 1,
-                     xoptions=gtk.FILL, yoptions=0)
-        self.g_inversion_box = gtk.VBox()
-        table.attach(self.g_inversion_box, 2, 3, 2, 3, xoptions=gtk.FILL)
-
-        self.g_toptone_label = gtk.Label(_("Toptone"))
-        table.attach(self.g_toptone_label, 4, 5, 0, 1,
-                     xoptions=gtk.FILL, yoptions=0)
-        self.g_toptone_box = gtk.VBox()
-        table.attach(self.g_toptone_box, 4, 5, 2, 3, xoptions=gtk.FILL)
-
         self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
         self.g_music_displayer.set_size_request(100, -1)
         hbox.pack_start(self.g_music_displayer, False)
@@ -185,8 +115,13 @@ class Gui(abstract.LessonbasedGui):
         self.g_select_questions.initialize(4, 0)
         category_box.pack_start(self.g_select_questions, False)
         self.g_select_questions.show()
-
     def update_select_question_buttons(self):
+        """
+        The g_select_questions widget is used in m_custom_mode to select which
+        questions to ask. This method will show and update the widget
+        to the current lesson file if we are in m_custom_mode. If not, it
+        will hide the widget.
+        """
         if self.m_t.m_custom_mode:
             self.g_select_questions_category_box.show()
             self.g_select_questions.initialize(self.m_t.m_P.header.fillnum,
@@ -198,47 +133,67 @@ class Gui(abstract.LessonbasedGui):
             self.g_select_questions_category_box.hide()
             self.g_select_questions.initialize(0, 0)
     def update_answer_buttons(self, obj=None):
-        for x in self.g_chordtype_box.get_children() \
-              + self.g_inversion_box.get_children() \
-              + self.g_toptone_box.get_children():
-            x.destroy()
-        if self.m_t.m_P:
-            for ctype in self.m_t.m_P.get_unique_cnames():
-                translated_type = self.m_t.m_P.m_chord_types[ctype]
-                b = gtk.Button(translated_type)
-                sr = b.size_request()
-                b.set_size_request(int(sr[0]*1.3), sr[1])
-                b.set_data('type', ctype)
-                b.connect('clicked', self.on_type)
-                b.connect('button_release_event', self.on_type)
-                self.g_chordtype_box.pack_start(b, False)
-                b.show()
-            v = self.m_t.m_P.m_inversions
-            v.sort()
-            for x in v:
-                if x == 0:
-                    s = _("root position")
-                else:
-                    s = _("%i. inversion") % x
-                b = gtk.Button(s)
-                sr = b.size_request()
-                b.set_size_request(int(sr[0]*1.3), sr[1])
-                b.set_data('inversion', x)
-                b.connect('clicked', self.on_inversion)
-                b.connect('button_release_event', self.on_inversion)
-                self.g_inversion_box.pack_start(b, False)
-                b.show()
-            v = self.m_t.m_P.m_toptones
-            v.sort()
-            for x in v:
-                b = gtk.Button(str(x))
-                b.set_data('toptone', x)
-                b.connect('clicked', self.on_toptone)
-                b.connect('button_release_event', self.on_toptone)
-                self.g_toptone_box.pack_start(b, False)
-                b.show()
-        self.g_inversion_box.set_sensitive(True)
-        self.g_toptone_box.set_sensitive(True)
+        """
+        Only columns with question properties that are actually used
+        in the lesson file will be displayed. This way, we can make a default
+        configuration:
+         qprops = "name", "toptone", "inversion"
+         qprop_labels = _("Name"), _("Toptone"), _("Inversion")
+        and only lesson files that require other properties have to define
+        these two variables.
+        """
+        # This part will create the table with the buttons used to answer.
+        try:
+            self.g_atable.destroy()
+        except AttributeError:
+            pass
+        self.g_atable = gtk.Table()
+        self.g_atable.show()
+        self.g_hbox.pack_start(self.g_atable, False)
+        self.g_hbox.reorder_child(self.g_atable, 1)
+        # pprops say how many properties are we going to display.
+        # We will not display a property if no questions use it.
+        num_used_props = len(
+            [x for x in self.m_t.m_P.m_props.keys() if self.m_t.m_P.m_props[x]])
+        # tcols say how many columns we need. We need a column for each
+        # column separator
+        tcols = num_used_props * 2 - 1
+        trows = max([len(x) for x in self.m_t.m_P.m_props.values()]) + 2
+        # The column headings
+        for idx, label in enumerate(self.m_t.m_P.header.qprop_labels):
+            if isinstance(label, unicode):
+                s = label
+            else:
+                s = label.i18name
+            self.g_atable.attach(gtk.Label(s), idx * 2, idx * 2 + 1, 0, 1, 
+                                 xoptions=gtk.FILL, yoptions=gtk.SHRINK,
+                                 xpadding=gu.PAD_SMALL)
+        # Then we create the buttons used to answer.
+        for x, prop in enumerate(self.m_t.m_P.header.qprops):
+            for y, proplabel in enumerate(self.m_t.m_P.m_props[prop]):
+                button = gtk.Button(unicode(proplabel))
+                button.set_data('property_name', prop)
+                button.set_data('property_value', proplabel.cval)
+                button.connect('clicked', self.on_prop_button_clicked)#, prop, self.m_t.m_P.m_props[prop][y])
+                self.g_atable.attach(button, x * 2, x * 2 + 1, y + 2, y + 3,
+                    xpadding=gu.PAD_SMALL,
+                    yoptions=gtk.SHRINK)
+        # The separator below the column headings
+        self.g_atable.attach(gtk.HSeparator(), 0, tcols, 1, 2,
+            xoptions=gtk.FILL, yoptions=gtk.FILL,
+            xpadding=0, ypadding=gu.PAD_SMALL)
+        # The vertical separator between columns
+        for idx in range(len(self.m_t.m_P.header.qprops)-1):
+            self.g_atable.attach(gtk.VSeparator(), 
+            idx * 2 + 1, idx * 2 + 2, 0, trows,
+            xoptions=gtk.FILL, yoptions=gtk.FILL,
+            xpadding=0, ypadding=gu.PAD_SMALL)
+        self.g_atable.show_all()
+        #
+        self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
+        self.g_repeat.set_sensitive(False)
+        self.g_repeat_arpeggio.set_sensitive(False)
+        self.g_give_up.set_sensitive(False)
     def update_gui_after_lessonfile_change(self):
         self.g_music_displayer.clear()
         self.update_select_question_buttons()
@@ -247,48 +202,27 @@ class Gui(abstract.LessonbasedGui):
         else:
             self.set_lesson_heading(_("Identify the chord"))
         self.g_new.set_sensitive(True)
-        self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
-        if not [q for q in self.m_t.m_P.m_questions if 'toptone' in q]:
-            self.g_toptone_label.hide()
-            self.g_vsep2.hide()
-        else:
-            self.g_toptone_label.show()
-            self.g_vsep2.show()
-        if not [q for q in self.m_t.m_P.m_questions if 'inversion' in q]:
-            self.g_inversion_label.hide()
-            self.g_vsep1.hide()
-        else:
-            self.g_inversion_label.show()
-            self.g_vsep1.show()
-        self.g_repeat.set_sensitive(False)
-        self.g_repeat_arpeggio.set_sensitive(False)
-        self.g_give_up.set_sensitive(False)
         self.update_answer_buttons()
+    def on_prop_button_clicked(self, button):
+        g = self.m_t.guess_property(button.get_data('property_name'),
+                                    button.get_data('property_value'))
+        if g:
+            self.g_flashbar.flash(_("Correct"))
+            for btn in self.g_atable.get_children():
+                if btn.get_data('property_name') == button.get_data('property_name')\
+                 and btn.get_data('property_value') == button.get_data('property_value'):
+                    btn.get_children()[0].set_name("BoldText")
+                    break
+            if g == self.m_t.ALL_CORRECT:
+                self.all_guessed_correct()
+        else:
+            self.g_flashbar.flash(_("Wrong"))
     def on_type(self, button, event=None):
         if event is None:
             self.on_type_left_clicked(button)
         elif event.button == 3:
             if self.m_t.m_P and self.m_t.m_P.header.enable_right_click:
                 self.on_type_right_clicked(button)
-    def on_type_left_clicked(self, button):
-        if self.m_t.q_status in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP)\
-          or self.m_t.m_correct[0]:
-            return
-        g = self.m_t.guess_type(button.get_data('type'))
-        if g:
-            self.g_flashbar.flash(_("Correct"))
-            for b in self.g_chordtype_box.get_children():
-                if b == button:
-                    b.get_children()[0].set_name("BoldText")
-            if g == self.m_t.ALL_CORRECT:
-                self.all_guessed_correct()
-        else:
-            self.g_flashbar.flash(_("Wrong"))
-        if self.m_t.m_correct == [1, 1, 1]:
-            self.run_exception_handled(self.show_answer)
-            self.g_new.set_sensitive(True)
-            self.g_new.grab_focus()
-            self.g_give_up.set_sensitive(False)
     def on_type_right_clicked(self, button):
         if self.m_t.q_status == const.QSTATUS_NO:
             return
@@ -335,22 +269,6 @@ class Gui(abstract.LessonbasedGui):
         elif event.button == 3:
             if self.m_t.m_P and self.m_t.m_P.header.enable_right_click:
                 self.on_inversion_right_clicked(button)
-    def on_inversion_left_clicked(self, button):
-        if self.m_t.q_status in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP) \
-          or self.m_t.m_correct[1]:
-            return
-        if self.m_t.guess_inversion(button.get_data('inversion')):
-            self.g_flashbar.flash(_("Correct"))
-            for b in self.g_inversion_box.get_children():
-                if b == button:
-                    b.get_children()[0].set_name("BoldText")
-        else:
-            self.g_flashbar.flash(_("Wrong"))
-        if self.m_t.m_correct == [1, 1, 1]:
-            self.run_exception_handled(self.show_answer)
-            self.g_new.set_sensitive(True)
-            self.g_new.grab_focus()
-            self.g_give_up.set_sensitive(False)
     def on_inversion_right_clicked(self, button):
         """
         First we try to find a chord with the same type and toptone as
@@ -402,24 +320,10 @@ class Gui(abstract.LessonbasedGui):
                     and question.get_toptone() == button.get_data('toptone'):
                 sel.run_exception_handled(self.m_t.m_P.play_question, question)
                 return
-    def on_toptone_left_clicked(self, button):
-        if self.m_t.q_status in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP) \
-           or self.m_t.m_correct[2]:
-            return
-        if self.m_t.guess_toptone(button.get_data('toptone')):
-            self.g_flashbar.flash(_("Correct"))
-            for b in self.g_toptone_box.get_children():
-                if b == button:
-                    b.get_children()[0].set_name("BoldText")
-        else:
-            self.g_flashbar.flash(_("Wrong"))
-        if self.m_t.m_correct == [1, 1, 1]:
-            self.run_exception_handled(self.show_answer)
-            self.g_new.set_sensitive(True)
-            self.g_new.grab_focus()
-            self.g_give_up.set_sensitive(False)
     def all_guessed_correct(self):
+        self.run_exception_handled(self.show_answer)
         self.g_new.set_sensitive(True)
+        self.g_new.grab_focus()
         self.g_give_up.set_sensitive(False)
     def new_question(self, widget=None):
         def exception_cleanup():
@@ -432,10 +336,10 @@ class Gui(abstract.LessonbasedGui):
         if not self.m_t.m_P:
             return
         # make sure all buttons are sensitive.
-        for button in self.g_chordtype_box.get_children() \
-                    + self.g_inversion_box.get_children() \
-                    + self.g_toptone_box.get_children():
-            button.get_children()[0].set_name("")
+        for x in self.g_atable.get_children():
+            if isinstance(x, gtk.Button):
+                # Set name to "" to make all labels not have bold text.
+                x.get_children()[0].set_name("")
         ##
         try:
             n = self.m_t.new_question()
@@ -449,9 +353,7 @@ class Gui(abstract.LessonbasedGui):
                 if self.get_bool('config/picky_on_new_question'):
                     self.g_new.set_sensitive(False)
                 self.g_repeat_arpeggio.set_sensitive(True)
-                self.g_inversion_box.set_sensitive(self.m_t.m_P.get_inversion() != -1)
-                self.g_toptone_box.set_sensitive(self.m_t.m_P.get_toptone() != -1)
-                self.g_chordtype_box.get_children()[0].grab_focus()
+                [btn for btn in self.g_atable.get_children() if isinstance(btn, gtk.Button)][-1].grab_focus()
         except Exception, e:
             if not self.standard_exception_handler(e, __file__,
                     exception_cleanup):
@@ -479,47 +381,14 @@ class Gui(abstract.LessonbasedGui):
         if self.m_t.q_status == const.QSTATUS_NEW:
             self.m_t.give_up()
             self.run_exception_handled(self.show_answer)
-            type_txt = self.m_t.m_P.m_chord_types[self.m_t.m_P.get_cname()]
-            i = self.m_t.m_P.get_inversion()
-            if i == 0:
-                inversion_txt = _("root position")
-            elif i > 0:
-                inversion_txt = _("%i. inversion") % i
-            else:
-                inversion_txt = ""
-            i = self.m_t.m_P.get_toptone()
-            if i != -1:
-                toptone_txt = "toptone: %i" % i
-            else:
-                toptone_txt = ""
-            v = [type_txt]
-            if inversion_txt:
-                v.append(inversion_txt)
-            if toptone_txt:
-                v.append(toptone_txt)
-            if len(v) == 3:
-                self.g_flashbar.flash(_("%(one)s, %(two)s and %(three)s") % {'one': v[0], 'two': v[1], 'three': v[2]})
-            elif len(v) == 2:
-                self.g_flashbar.flash(_("%(one)s and %(two)s") % {'one': v[0], 'two': v[1]})
-            else:
-                self.g_flashbar.flash(type_txt)
             self.g_new.set_sensitive(True)
             self.g_give_up.set_sensitive(False)
-            for button in self.g_chordtype_box.get_children():
-                if button.get_data('type') == self.m_t.m_P.get_cname():
-                    button.get_children()[0].set_name('BoldText')
-                else:
-                    button.get_children()[0].set_name('')
-            for button in self.g_inversion_box.get_children():
-                if button.get_data('inversion') == self.m_t.m_P.get_inversion():
-                    button.get_children()[0].set_name('BoldText')
-                else:
-                    button.get_children()[0].set_name('')
-            for button in self.g_toptone_box.get_children():
-                if button.get_data('toptone') == self.m_t.m_P.get_toptone():
-                    button.get_children()[0].set_name('BoldText')
-                else:
-                    button.get_children()[0].set_name('')
+            for button in self.g_atable.get_children():
+                if isinstance(button, gtk.Button):
+                    if button.get_data('property_value') == self.m_t.m_P.get_question()[button.get_data('property_name')].cval:
+                        button.get_children()[0].set_name('BoldText')
+                    else:
+                        button.get_children()[0].set_name('')
     def show_answer(self):
         """
         Show the answer in the music displayer. All callers must check
