@@ -309,11 +309,11 @@ class Gui(abstract.LessonbasedGui):
         for x in self.g_chordtype_box.get_children():
             x.destroy()
         if self.m_t.m_P:
-            for ctype, translated_type in self.m_t.m_P.m_chord_types.iteritems():
-                b = gtk.Button(translated_type)
-                b.set_data('chordtype', ctype)
+            for ctype in self.m_t.m_P.m_props['name']:
+                b = gtk.Button(ctype)
+                b.set_data('chordtype', ctype.cval)
                 self.g_chordtype_box.pack_start(b, False)
-                b.connect('clicked', self.on_chordtype_clicked, ctype)
+                b.connect('clicked', self.on_chordtype_clicked, ctype.cval)
                 b.connect('button_release_event', self.on_chordtype_right_clicked, ctype)
                 b.show()
     def set_chordtype_frame_status_solved(self, t):
