@@ -249,41 +249,79 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         else:
             self.create_linux_sound_page()
         self.show_all()
+    def add_gui_for_external_programs(self, page_vbox):
+        box, category_vbox = gu.hig_category_vbox(_("External programs"))
+        page_vbox.pack_start(box, False)
+        if sys.platform != 'win32':
+            format_info = [
+                {'id': 'wav',
+                 # testfile is a file in lesson-files/share
+                 'testfile': 'fifth-small-220.00.wav',
+                 'label': _(".wav file player"),
+                 # players is a tuple of tuples. The inner tuple has to elements:
+                 # '/path/to/player', 'comandline'
+                 'players': (('/usr/bin/play', '/usr/bin/play %s'),
+                             ('/usr/bin/aplay', '/usr/bin/aplay %s'),
+                             ('/usr/bin/esdplay', '/usr/bin/esdplay %s'),
+                            ),
+                }
+            ]
+        else:
+            format_info = []
+        format_info.extend([
+            {'id': 'midi',
+             'testfile': 'fanfare.midi',
+             'label': _(".midi file player"),
+             'players': (('/usr/bin/timidity', '/usr/bin/timidity -idqq %s'),
+                         ('/usr/bin/drvmidi', '/usr/bin/drvmidi'),
+                         ('/usr/bin/playmidi', '/usr/bin/playmidi'),
+                        ),
+            },
+            {'id': 'mp3',
+             'testfile': 'fanfare.mp3',
+             'label': _("MP3 file player"),
+             'players': (('/usr/bin/mpg123', '/usr/bin/mpg123'),
+                         ('/usr/bin/alsaplayer', '/usr/bin/alsaplayer'),
+                        ),
+            },
+            {'id': 'ogg',
+             'testfile': 'fanfare.ogg',
+             'label': _("OGG file player"),
+             'players': (('/usr/bin/ogg123', '/usr/bin/ogg123'),
+                         ('/usr/bin/alsaplayer', '/usr/bin/alsaplayer'),
+                        ),
+            },
+        ])
+        self.g_player_boxes = {}
+        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        for format in format_info:
+            liststore = gtk.ListStore(gobject.TYPE_STRING)
+            for path, cmd in format['players']:
+                if os.path.exists(path):
+                    liststore.append((cmd,))
+            self.g_player_boxes[format['id']] = gtk.ComboBoxEntry(liststore)
+            self.g_player_boxes[format['id']].child.set_text(cfg.get_string('sound/%s_player' % format['id']))
+            def _changed(widget, formatid):
+                self.set_string('sound/%s_player' % formatid,
+                                widget.child.get_text())
+            self.g_player_boxes[format['id']].connect('changed',
+                _changed, format['id'])
+            button = gtk.Button(_("Test"))
+            button.connect('clicked', self.test_XXX_player,
+                           format['id'], format['testfile'])
+            box = gu.hig_label_widget(format['label'],
+                                      [self.g_player_boxes[format['id']], button],
+                                      sizegroup)
+            category_vbox.pack_start(box)
+    def test_XXX_player(self, w, typeid, testfile):
+        osutils.run_external_program(self.g_player_boxes[typeid].child.get_text(),
+                 'lesson-files/share', testfile)
     def create_linux_sound_page(self):
         page_vbox = gu.hig_dlg_vbox()
         self.g_notebook.append_page(page_vbox, gtk.Label(_("Sound setup")))
 
-        box, category_vbox = gu.hig_category_vbox(_("External programs"))
-        page_vbox.pack_start(box, False)
-
-        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        for n in find_wav_player_programs():
-            liststore.append((n,))
-        self.g_wav_player = gtk.ComboBoxEntry(liststore)
-        self.g_wav_player.child.set_text(cfg.get_string('sound/wav_player'))
-        self.g_wav_player.connect('changed',
-            lambda w: self.set_string('sound/wav_player', w.child.get_text()))
-        button = gtk.Button(_("Test"))
-        button.connect('clicked', self.test_wav_player)
-        box = gu.hig_label_widget(_(".wav file player"),
-                                  [self.g_wav_player, button],
-                                  sizegroup)
-        category_vbox.pack_start(box)
-
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        for s in find_midi_player_programs():
-            liststore.append((s,))
-        self.g_midi_player = gtk.ComboBoxEntry(liststore)
-        self.g_midi_player.child.set_text(cfg.get_string('sound/midi_player'))
-        self.g_midi_player.connect('changed',
-            lambda w: self.set_string('sound/midi_player', w.child.get_text()))
-        button = gtk.Button(_("Test"))
-        button.connect('clicked', self.test_midi_player)
-        box = gu.hig_label_widget(_(".midi file player"),
-                                  [self.g_midi_player, button],
-                                  sizegroup)
-        category_vbox.pack_start(box)
+        ###
+        self.add_gui_for_external_programs(page_vbox)
 
         #############
         # midi setup
@@ -343,23 +381,6 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         page_vbox = gu.hig_dlg_vbox()
         self.g_notebook.append_page(page_vbox, gtk.Label(_("Midi setup")))
 
-        box, category_vbox = gu.hig_category_vbox(_("External programs"))
-        page_vbox.pack_start(box, False)
-
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        for s in ('/usr/bin/timidity -idqq %s', '/usr/bin/playmidi -f %s'):
-            liststore.append((s, ))
-        self.g_midi_player = gtk.ComboBoxEntry(liststore)
-        self.g_midi_player.child.set_text(cfg.get_string('sound/midi_player'))
-        self.g_midi_player.connect('changed',
-            lambda w: self.set_string('sound/midi_player', w.child.get_text()))
-        button = gtk.Button(_("Test"))
-        button.connect('clicked', self.test_midi_player)
-        box = gu.hig_label_widget(".midi file player",
-                                  [self.g_midi_player, button],
-                                  None)
-        category_vbox.pack_start(box)
-
         #############
         # midi setup
         #############
@@ -396,13 +417,6 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             self.g_fakesynth_radio.set_active(True)
 
         gu.bButton(category_vbox, _("_Apply changes and play test sound"), self.on_apply_and_play_test_sound)
-
-    def test_wav_player(self, w):
-        osutils.run_external_program(self.g_wav_player.child.get_text(),
-                 'lesson-files/share', 'fifth-small-220.00.wav')
-    def test_midi_player(self, w):
-        osutils.run_external_program(self.g_midi_player.child.get_text(),
-                        'lesson-files/share', 'fanfare.midi')
     def set_gui_from_config(self):
         if self.get_string("sound/type") == "fake-synth":
             self.g_fakesynth_radio.set_active(True)
@@ -443,7 +457,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             soundcard.synth.close()
         if self.g_midiplayer_radio.get_active():
             soundcard.initialise_external_midiplayer(
-                  self.g_midi_player.child.get_text())
+                  self.g_player_boxes['midi'].child.get_text())
             soundcard.synth.error_report_cb = self.m_app.m_ui.display_error_message
         elif self.g_device_radio.get_active():
             if sys.platform != 'win32' and configureoutput.HAVE_LINUX_AWE_VOICE_H and \
@@ -483,9 +497,4 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             # we set the spin just in case m_devnum was changed by the
             # soundcard setup code, if it was out of range
             self.g_synth_num.set_value(soundcard.synth.m_devnum)
-        if sys.platform != 'win32':
-            self.set_string("sound/wav_player",
-                            self.g_wav_player.child.get_text())
-        self.set_string("sound/midi_player",
-                        self.g_midi_player.child.get_text())
 
