@@ -91,19 +91,42 @@ TOKEN_STRING = 1
 TOKEN_IDX = 2
 TOKEN_LINENO = 3
 
-class _TranslatedString:
-    def __init__(self, i18name, cname):
-        self.i18name = i18name
-        self.cname = cname
+class istr(unicode):
+    def __init__(self, s):
+        self.cval = s
+        self.m_added_language = None
+    def add_translation(self, lang, s):
+        """
+        Use this method to add translations that are included directly in
+        the lesson file like this:
+
+          name = "major"
+          name[no] = "dur"
+        """
+        if lang in i18n.langs():
+            # i18n.langs() has a list of the langauges we can use.
+            # The first language in the list is preferred.
+            new_pos = i18n.langs().index(lang)
+            if not self.m_added_language:
+                old_pos = sys.maxint
+            else:
+                old_pos = i18n.langs().index(self.m_added_language)
+            if new_pos < old_pos:
+                retval = istr(s)
+                retval.m_added_language = lang
+                retval.cval = self.cval
+                return retval
+        return self
+    def new_translated(cval, translated):
+        retval = istr(translated)
+        retval.cval = cval
+        return retval
+    new_translated = staticmethod(new_translated)
 
 def dataparser_i18n_func(s):
-    return _TranslatedString(_(s), s)
-
-def get_translated_string(d, name):
-    for n in i18n.langs():
-        if "%s[%s" % (name, n) in d:
-            return d["%s[%s]" % (name, n)]
-    return d[name]
+    retval = istr(_(s))
+    retval.cval = s
+    return retval
 
 class Question(dict):
     def get_toptone(self):
@@ -125,18 +148,13 @@ class Question(dict):
         return self['name']
     def get_name(self):
         """
-        Return the translated name of the question, or the untranslated
-        if not available. name[no]="blabla" will be used before
-        name=_("blabla") if both are available.
+        Return the name of the question. Return "[no name]" if the question
+        is missing the name variable.
+        FIXME fortsett her. Trenger vi denne metoden i det hele tatt.
         """
-        for n in i18n.langs():
-            if "name[%s]" % n in self:
-                return self["name[%s]" % n]
         if "name" in self:
             return self["name"]
-        if "C-name" in self:
-            return self["C-name"]
-        return "[no name]"
+        return istr("[no name]")
 
 
 class DataparserException(Exception):
@@ -180,6 +198,14 @@ class AssignmentToReservedWordException(DataparserException):
         self.m_token = parser._lexer.m_tokens[bad_pos]
         self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
 
+class CannotTranslateListsException(DataparserException):
+    def __init__(self, parser, bad_pos, variable):
+        DataparserException.__init__(self, _("Cannot translate lists using infile translations (ex var[no]=...). See the variable \"%(variable)s\" in the file \"%(filename)s\"") % {'filename': parser.m_filename, 'variable': variable})
+        # This variable is only used by the module test code.
+        self.m_token = parser._lexer.m_tokens[bad_pos]
+        self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
+
+
 class UnableToTokenizeException(DataparserException):
     def __init__(self, lexer, lineno, token, pos):
         """
@@ -319,6 +345,8 @@ class Dataparser:
         self.context = self.globals
         self.m_filename = None
         self.m_ignore_lookup_error = False
+        self.m_translation_re = re.compile("(?P<varname>\w+)\[(?P<lang>[\w_+]+)\]")
+
     def parse_file(self, filename):
         """We always construct a new parser if we want to parse another
         file. So this method is never called twice for one parser.
@@ -405,14 +433,31 @@ class Dataparser:
             raise AssignmentToReservedWordException(self, npos, name)
         self._lexer.scan_any()#('=')
         faktorlist = self.faktorlist()
-        if len(faktorlist) == 1:
-            if isinstance(faktorlist[0], _TranslatedString):
-                self.context[name] = faktorlist[0].i18name
-                self.context["C-%s" % name] = faktorlist[0].cname
+        m = self.m_translation_re.match(name)
+        if m:
+            if len(faktorlist) != 1:
+                raise CannotTranslateListsException(self, npos, name)
+            faktor = faktorlist[0]
+            assert type(faktor) == istr
+            if m.group('varname') in self.context:
+                self.context[m.group('varname')] = self.context[m.group('varname')].add_translation(m.group('lang'), faktor)
             else:
-                self.context[name] = faktorlist[0]
+                # add the first translation as cval until we get the correct
+                # value to use.
+                self.context[m.group('varname')] = faktor
+                # Also add faktor as a translation, since it might be the
+                # translation we need.
+                self.context[m.group('varname')] = self.context[m.group('varname')].add_translation(m.group('lang'), faktor)
         else:
-            self.context[name] = faktorlist
+            # We only check for cval if len(faktorlist) == 1, because
+            # lists are not localized.
+            if len(faktorlist) == 1:
+                if name in self.context and isinstance(self.context[name], istr):
+                    self.context[name].cval = faktorlist[0]
+                else:
+                    self.context[name] = faktorlist[0]
+            else:
+                self.context[name] = faktorlist
     def faktor(self):
         """faktor: atom
               ("+" atom
@@ -425,10 +470,7 @@ class Dataparser:
         while 1:
             if peek == '+':
                 self._lexer.scan_any()
-                if isinstance(faktor, _TranslatedString):
-                    faktor = faktor.i18name % self.atom()
-                else:
-                    faktor += self.atom()
+                faktor += self.atom()
             elif peek == '-':
                 self._lexer.scan_any()
                 faktor -= self.atom()
@@ -437,10 +479,7 @@ class Dataparser:
                 faktor = (faktor, self.atom())
             elif peek == '%':
                 self._lexer.scan_any()
-                if isinstance(faktor, _TranslatedString):
-                    faktor = faktor.i18name % self.atom()
-                else:
-                    faktor = faktor % self.atom()
+                faktor = faktor % self.atom()
             else:
                 break
             peek = self._lexer.peek_type()
@@ -457,7 +496,7 @@ class Dataparser:
         npos = self._lexer.pos
         peek = self._lexer.peek_type()
         if peek == 'STRING':
-            return self._lexer.scan('STRING')
+            return istr(self._lexer.scan('STRING'))
         elif peek == 'INTEGER':
             return int(self._lexer.scan('INTEGER'))
         elif peek == 'FLOAT':
@@ -494,7 +533,7 @@ class Dataparser:
                 return self.functions[name](*arglist)
             except KeyError:
                 raise NameLookupException(self, npos)
-            except TypeError:
+            except TypeError, e:
                 raise WrongArgumentCount(self, npos)
     def block(self):
         """block: NAME "{" assignmentlist "}" """
