@@ -60,7 +60,14 @@ def langs():
 
 def setup(prefix, config_locale=None):
     # Gettext and gtk+ chech the variables in this order.
-    locale.setlocale(locale.LC_ALL, '')
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+    except locale.Error, e:
+        print "=" * 60
+        print "The following traceback is probably caused by bad setting of"
+        print "LC_XXX variables. See the Solfege README file."
+        print "=" * 60
+        raise e
     varlist = ('LANGUAGE', 'LC_MESSAGES')
     if not config_locale:
         config_locale = 'system default'
