@@ -188,13 +188,13 @@ class Gui(abstract.LessonbasedGui):
         self.g_display_music.set_sensitive(False)
         self.g_play_music.set_sensitive(False)
         self.g_give_up.set_sensitive(False)
-        if filter(lambda q: q.is_transposable(), self.m_t.m_P.m_questions):
+        if filter(lambda q: q['music'].is_transposable(), self.m_t.m_P.m_questions):
             self.g_random_transpose_box.show()
         else:
             self.g_random_transpose_box.hide()
         # If one or more of the questions is of musictype 'chord', then
         # we need the "Repeat arpeggio" button.
-        if filter(lambda q: q['music'].m_musictype == 'chord',
+        if filter(lambda q: isinstance(q['music'], lessonfile.Chord),
                   self.m_t.m_P.m_questions):
             self.g_repeat_arpeggio.show()
         else:
