@@ -65,6 +65,7 @@ import utils
 from mpd.rat import Rat
 import soundcard
 import dataparser
+from dataparser import istr
 import osutils
 
 class plstring(unicode):
@@ -175,6 +176,8 @@ lessonfile_functions = {
         'cmdline': lambda m: Cmdline(m),
         'wavfile': lambda m: Wavfile(m),
         'midifile': lambda m: Midifile(m),
+        'mp3file': lambda m: Mp3file(m),
+        'oggfile': lambda m: Oggfile(m),
         'progressionlabel': lambda s: plstring(s),
 }
 
@@ -196,6 +199,11 @@ class _Header(dict):
                            ('have_music_displayer', False),
                            ('enable_right_click', True),
                            ('disable_unused_intervals', True),
+                           ('qprops', ['name', 'inversion', 'toptone']),
+                           ('qprop_labels', [
+                             istr.new_translated("Chord type", _("Chord type")),
+                             istr.new_translated("Inversion", _("Inversion")),
+                             istr.new_translated("Toptone", _("Toptone"))]),
                            ):
             if key not in self:
                 self[key] = value
@@ -210,10 +218,8 @@ class _Header(dict):
             header['variable_name']
         """
         if name in self:
-            return dataparser.get_translated_string(self, name)
-        # FIXME print "warning: lessonfile.Header.%s not set. Returning empty string" % name
-        return ""
-        #raise NameError(name)
+            return self[name]
+        return istr("")
 
 
 class MusicBaseClass(object):
@@ -530,31 +536,35 @@ class Percussion(_PercBaseClass):
     def play(self, lessonfile_ref, question):
         soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
 
-
-class Midifile(MusicBaseClass):
-    def __init__(self, musicdata):
+class _MusicExternalPlayer(MusicBaseClass):
+    def __init__(self, typeid, musicdata):
         MusicBaseClass.__init__(self, musicdata)
+        self.m_typeid = typeid
     def play(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        midifile = os.path.join(lessonfile_ref.m_location, self.m_musicdata)
-        if os.path.exists(midifile):
-            soundcard.play_midi_file(midifile)
+        musicfile = os.path.join(lessonfile_ref.m_location, self.m_musicdata)
+        if os.path.exists(musicfile):
+            soundcard.play_mediafile(self.m_typeid, musicfile)
         else:
-            raise FileNotFound(midifile)
+            raise FileNotFound(musicfile)
 
-
-class Wavfile(MusicBaseClass):
+class Midifile(_MusicExternalPlayer):
     def __init__(self, musicdata):
-        MusicBaseClass.__init__(self, musicdata)
-    def play(self, lessonfile_ref, question):
-        assert isinstance(lessonfile_ref, LessonfileCommon)
-        wavfile = os.path.join(lessonfile_ref.m_location, self.m_musicdata)
-        if os.path.exists(wavfile):
-            soundcard.play_wav_file(wavfile)
-        else:
-            raise FileNotFound(wavfile)
+        _MusicExternalPlayer.__init__(self, 'midi', musicdata)
 
 
+class Wavfile(_MusicExternalPlayer):
+    def __init__(self, musicdata):
+        _MusicExternalPlayer.__init__(self, 'wav', musicdata)
+
+class Mp3file(_MusicExternalPlayer):
+    def __init__(self, musicdata):
+        _MusicExternalPlayer.__init__(self, 'mp3', musicdata)
+
+class Oggfile(_MusicExternalPlayer):
+    def __init__(self, musicdata):
+        _MusicExternalPlayer.__init__(self, 'ogg', musicdata)
+
 class Cmdline(MusicBaseClass):
     def __init__(self, musicdata):
         MusicBaseClass.__init__(self, musicdata)
@@ -790,7 +800,7 @@ class QuestionsLessonfile(LessonfileCommon):
         """
         names = {}
         for question in self.m_questions:
-            if question.get_cname() not in names:
+            if 'name' in question and question['name'].cval not in names:
                 names[question.get_cname()] = 1
                 yield question
     def get_unique_cnames(self):
@@ -800,7 +810,7 @@ class QuestionsLessonfile(LessonfileCommon):
         """
         names = []
         for question in self.m_questions:
-            if not question.get_cname() in names:
+            if 'name' in question and question['name'].cval not in names:
                 names.append(question.get_cname())
         return names
     def get_question(self):
@@ -1115,26 +1125,51 @@ class ChordLessonfile(QuestionsLessonfile):
         self.parse_file(filename)
     def parse_file(self, filename):
         """
-        Call ChordLessonfile.parse_file and after that set these variables:
-        self.m_chord_types
-        self.m_inversions
-        self.m_toptone
+        Call ChordLessonfile.parse_file and set the self.m_props dict.
+        Change some question variables, so that:
+
+          inversion = 0
+
+        is the same as
+
+          inversion = _("root position")
         """
         super(ChordLessonfile, self).parse_file(filename)
-        self.m_chord_types = {}
-        self.m_inversions = []
-        self.m_toptones = []
+        if len(self.header.qprops) != len(self.header.qprop_labels):
+            raise LessonfileParseException(_("Error in the lesson file header of \"%(filename)s\". The variables qprops and qprop_labels has to have the same length.") % {'filename': self.m_filename})
+        # m_props will be a dict where each key is the property var name.
+        # The values will be a list of possible values for that property.
+        # The values are of type istr. This mean that .cval holds the
+        # C locale string.
+        self.m_props = {}
+        for k in self.header.qprops:
+            self.m_props[k] = []
         for question in self.m_questions:
-            self.m_chord_types[question.get_cname()] = question.get_name()
-
-            # inversion
-            if 'inversion' in question \
-                    and question['inversion'] not in self.m_inversions:
-                self.m_inversions.append(question['inversion'])
-            # toptone
-            if 'toptone' in question \
-                    and question['toptone'] not in self.m_toptones:
-                self.m_toptones.append(question['toptone'])
+            for varname in self.header.qprops:
+                if varname in question:
+                    if varname == 'inversion':
+                        if question[varname] == 0:
+                            question[varname] = istr(_("root position"))
+                            question[varname].cval= "root position"
+                        elif type(question[varname]) == int \
+                            and question[varname] > 0:
+                            i = question[varname]
+                            question[varname] = istr(_("%i. inversion") % i)
+                            question[varname].cval = "%i. inversion" % i
+                    # FIXMECOMPAT convert integer properties to strings.
+                    # This to be compatible with solfege 3.9.1 and older.
+                    if type(question[varname]) in (int, float):
+                        question[varname] = istr(unicode(question[varname]))
+                    # then add to m_props
+                    if question[varname] not in self.m_props[varname]:
+                        self.m_props[varname].append(question[varname])
+        for k in self.m_props:
+            if not self.m_props[k]:
+                idx = self.header.qprops.index(k)
+                del self.header.qprops[idx]
+                del self.header.qprop_labels[idx]
+        for k in [k for k in self.m_props if not self.m_props[k]]:
+            del self.m_props[k]
     def get_inversion(self):
         """
         Return the inversion of the currently selected question.
