@@ -47,7 +47,7 @@ class Gui(abstract.LessonbasedGui):
         self.g_new = gu.bButton(self.action_area, _("_New"), self.new_question)
         gu.bButton(self.action_area, _("440h_z"), lambda f, s=self: s.m_t.play_440hz())
         self.g_play_answer = gu.bButton(self.action_area, _("_Play answer"),
-            lambda w: self.run_exception_handled(self.m_t.m_P.play_question_musicformat_satb_arpeggio))
+            lambda w: self.run_exception_handled(self.m_t.m_P.play_question_arpeggio))
         self.practise_box.show_all()
         ##############
         # config_box #
